require 'iowa'
require 'fcgi'

module Iowa

	def Iowa.handleRequest(request)
		deployed_screen = false
		specific_page = nil
		buffer = nil
		original_exception = nil
		loop do
			request.content_type = 'text/html'
			buffer = ''
			context = nil
			exception = nil
			exception = catch(:session_error) do
				begin
					context = Iowa::Context.new(request, buffer)
					context.prior_exception = original_exception if original_exception
					app.handleRequest(context,specific_page)
				rescue Exception => exception
				end
			end

			if exception.to_s != '' and exception.kind_of?(Exception)
				mylog = Logger['iowa_log']
				mylog.warn "Execution Error: #{exception} :: " + exception.backtrace.join(".....").to_s unless original_exception
				if internal_host?(request)
					mylog.info "Showing PrettyException"
					buffer << Iowa::PrettyException.new(exception).to_s
				else
					if deployed_screen == false and specific_page = checkScreen(exception)
						mylog.info "Deploying exception screen #{specific_page}"
						request.status_line = '500 Internal Server Error'
						original_exception = exception
						deployed_screen = true
						retry
					else
						mylog.info "Showing ugly exception"
						request.status_line = '500 Internal Server Error'
						exception = original_exception if original_exception
						buffer << "<p> An unhandled error has occured.  Please inform your system or your site's administrator of the error.</p>"
						buffer << "<p>#{exception}<br/>"
						buffer << exception.backtrace.join("<br/>\n").to_s
						buffer << "</p>"
						status = 'EOUT'
					end
				end
			end
			break
		end
		buffer
	end

	# Handle the communications coming in on the monitored socket,
	# create a context object from the data received, and then pass
	# the context information into the Application object for final
	# handling.  Exception handling is simply via capturing the Exception
	# and outputting a stack backtrace (this could be improved).

	def Iowa.handleConnection(req)
		begin
			Thread.current.priority = 2
			start_time = Time.now
			mylog = Logger['iowa_log']
			# Having two logfiles with much the same data in them is stupid.  Fix this.
#			myapachelikelog = Logger['iowa_apachelike_log']
			request = Iowa::Request.new(req)
			request.content_type ||= 'text/html'
			
			read_time = Time.now
			status = 'OK'
		rescue Exception => exception
			mylog.error "Failure While Reading Inbound Data: #{exception}"
			status = "EIN"
		end

		begin
			request = Marshal.load(message)
		rescue Exception => exception
			mylog.error "Inbound Data Corruption: #{exception}"
			status = 'EIN'
		end

		buffer = handleRequest(request)

		begin
			req.out.puts "Date: #{Time.now.asctime}"
			req.out.puts "Pragma: no_cache"
			req.out.puts "Connection: close"
			req.out.puts "Content-type: #{request.content_type}"
			req.out.puts "Content-Length: #{buffer.length}"
			request.headers_out.each {|k,v| req.out.puts "#{k.capitalize}: #{v}"}
			req.out.print "\r\n"
			req.out.print buffer
			req.finish
			end_time = Time.now
		rescue Exception => exception
			mylog.error "Failure While Writing Outbound Data: #{exception}"
		end

#		vhost = nil
#		request.unparsed_uri =~ /http:\/\/([^\/]*)\//
#		vhost = $1
#		referer = request.headers_in['Referer'] != '' ? request.headers_in['Referer'] : request.uri
#		myapachelikelog.info "#{request.remote_host} #{request.hostname} - [#{Time.now.strftime('%d/%b/%Y:%H:%M:%S')}] #{buffer.length} \"#{request.request_method} #{request.unparsed_uri} HTTP/1.1\" 200 #{buffer.length} \"#{referer}\" \"#{request.headers_in['User-Agent']}\" \"#{request.headers_in['Cookie']}\""
		mylog.info "#{start_time} (#{read_time.to_f - start_time.to_f}/#{end_time.to_f - start_time.to_f}) :: #{request.uri} #{status} #{buffer.length}B"
	end

	def eventLoop(socket)
		Logger['iowa_log'].info 'Entering the main FCGI event loop...'
		FCGI::Server.new(socket).each_request do |request|
			Thread.start {handleConnection(request)}	
		end
	end

end
